-- This file and its contents are licensed under the Apache License 2.0.
-- Please see the included NOTICE for copyright information and
-- LICENSE-APACHE for a copy of the license.

\ir setup.v9.sql
\ir setup.sparse_index.sql
\ir setup.fix_sparse_index_migration.sql
